<?php
namespace RealID;

use RealID\ApiClient;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Controller;
use Psr\Log\LoggerInterface;

class Api extends WP_REST_Controller { 

  public function register_routes() {
    $version = '1';
    $namespace = 'real-id/v' . $version;

    register_rest_route( $namespace, '/license/activate', array(
        'methods' => 'POST',
        'callback' => array($this, 'activate_license_key'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        },
        'args' => [
          'licenseKey' => [
            'type' => 'string',
            'required' => true,
            'validate_callback' => function($param, $request) {
              return isset($param) && is_string($param);
            },
          ]
        ]
      ) 
    );

    register_rest_route( $namespace, '/license/deactivate', array(
        'methods' => 'DELETE',
        'callback' => array($this, 'deactivate_license_key'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/shop', array(
        'methods' => 'GET',
        'callback' => array($this, 'get_shop'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/shop/settings', array(
        'methods' => 'POST',
        'callback' => array($this, 'update_settings'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/shop/update-delivery-methods', array(
        'methods' => 'POST',
        'callback' => array($this, 'update_delivery_methods'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/checks', array(
        'methods' => 'POST',
        'callback' => array($this, 'create_check'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/checks', array(
        'methods' => 'GET',
        'callback' => array($this, 'list_checks'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/checks/(?P<id>[a-zA-Z0-9-_]+)', array(
        'methods' => 'GET',
        'callback' => array($this, 'get_check'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/checks/(?P<id>[a-zA-Z0-9-_]+)/reminder', array(
        'methods' => 'POST',
        'callback' => array($this, 'deliver_check_reminder'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/checks/(?P<id>[a-zA-Z0-9-_]+)/approve', array(
        'methods' => 'POST',
        'callback' => array($this, 'manually_approve_check'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/checks/(?P<id>[a-zA-Z0-9-_]+)/reject', array(
        'methods' => 'POST',
        'callback' => array($this, 'manually_reject_check'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/checks/(?P<id>[a-zA-Z0-9-_]+)/data', array(
        'methods' => 'DELETE',
        'callback' => array($this, 'delete_check'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    // called from the frontend script created in the "thank you" confirmation page within the plugin
    register_rest_route( $namespace, '/check/order/associate', array(
        'methods' => 'POST',
        'callback' => array($this, 'associate_check_with_order'),
        'permission_callback' => '__return_true'
      ) 
    );

    register_rest_route( $namespace, '/shop/email-signature/add', array(
        'methods' => 'POST',
        'callback' => array($this, 'add_email_sender_signature'),

        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        },
        'args' => [
          'fromName' => [
            'type' => 'string',
            'required' => true,
            'validate_callback' => function($param, $request) {
              return isset($param) && is_string($param);
            },
          ],
          'fromEmail' => [
            'type' => 'string',
            'required' => true,
            'validate_callback' => function($param, $request) {
              return isset($param) && is_string($param);
            },
          ]
        ]
      ) 
    );

    register_rest_route( $namespace, '/shop/email-signature/check-verification-status', array(
        'methods' => 'POST',
        'callback' => array($this, 'check_email_sender_signature_verification_status'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/shop/email-signature/remove', array(
        'methods' => 'DELETE',
        'callback' => array($this, 'remove_email_sender_signature'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/shop/reset', array(
        'methods' => 'POST',
        'callback' => array($this, 'reset_to_sandbox_mode'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );

    register_rest_route( $namespace, '/license-key', array(
        'methods' => 'GET',
        'callback' => array($this, 'get_current_license_key'),
        'permission_callback' => function () {
          return current_user_can( 'manage_options' );
        }
      ) 
    );
  }

  /**
   * Return the license key depending on if the shop is in test or live mode
   * 
   * @return string
   */
  public function get_license_key() {
    $licenseKey = get_option('real_id_license_key');
    if($licenseKey) {
      return $licenseKey;
    }

    $sandboxLicenseKey = get_option('real_id_sandbox_license_key');
    return $sandboxLicenseKey;
  }

  /**
   * Retrieve the RVshare API client
   * 
   * @param $authenticated Boolean 
   * @return ApiClient
   */
  public function api($authenticated = true) {
    if(!$authenticated) {
      return new ApiClient();
    }

    return new ApiClient($this->get_license_key());
  }

  public function get_shop(WP_REST_Request $request) {
    // get shop settings
    $client = $this->api();

    $res = $client->client->get('/api/shop');
    $data = json_decode((string) $res->getBody());

    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());



    if($res->getStatusCode() === 401) {
      $response->set_data($client->getOptions());
    }

    return $response;
  }

  public function create_check(WP_REST_Request $request) {
    // create a new ID check
    $client = $this->api();
    $body = $request->get_params();

    // the API needs to know this an WC check, not a Shopify check.
    $body = array_merge($body, [ 'platform' => 'wc' ]);

    $res = $client->client->post('/api/checks/create', array( 'json' => $body ));
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response;
  }

  public function get_check($data) {
    // get the ID check
    $client = $this->api();

    $res = $client->client->get('/api/checks/'.$data['id']);
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response;
  }

  public function delete_check($data) {
    // get the ID check
    $client = $this->api();

    $res = $client->client->delete('/api/checks/'.$data['id'].'/data');
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response;
  }

  public function list_checks(WP_REST_Request $request) {
    // checks index page
    $client = $this->api();
    $params = $request->get_query_params();

    $res = $client->client->get('/api/checks', array( 'query' => $params ));

    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response;
  }

  public function update_settings(WP_REST_Request $request) {
    // update shop settings
    $client = $this->api();
    $body = $request->get_params();

    $client->client->post('/api/shop/settings', array( 'json' => $body ));
    // this is a hack, but the current settings also need to include the sandbox mode flag
    $res = $client->client->get('/api/shop');
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response;
  }

  public function update_delivery_methods(WP_REST_Request $request) {
    // update shop settings on the main API
    $client = $this->api();
    $body = $request->get_params();

    // set the local option
    update_option('real_id_delivery_methods', $body['deliveryMethods']);

    $res = $client->client->post('/api/shop/settings', array( 'json' => $body ));
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response;
  }

  public function activate_license_key(WP_REST_Request $request) {
    $licenseKey = $request['licenseKey'];
    $client = new ApiClient($licenseKey);
    $body = array( 'url' => home_url() );

    $res = $client->client->post('/api/shop/license/'.$licenseKey.'/activate', array( 'json' => $body ));
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    if($res->getStatusCode() == 200) {
      // store the license key locally so we can make authenticated requests
      if(real_id_option_exists('real_id_license_key')) {
        update_option('real_id_license_key', $licenseKey);
      } else {
        add_option('real_id_license_key', $licenseKey);
      }

      update_option('real_id_sandbox_mode', $data->sandboxMode);
    }
    
    return $response;
  }

  public function deactivate_license_key(WP_REST_Request $request) {
    $currentLicenseKey = get_option('real_id_license_key');
    $client = $this->api();

    $res = $client->client->delete('/api/shop/license/'.$currentLicenseKey.'/deactivate');
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    if($res->getStatusCode() == 200) {
      delete_option('real_id_license_key');
      update_option('real_id_sandbox_mode', true);
    }


    return $response;
  }

  public function deliver_check_reminder(WP_REST_Request $request) {
    // get the ID check
    $client = $this->api();
    $params = $request->get_params();
    $id = $params['id'];

    $res = $client->client->post('/api/checks/'.$id.'/deliver/reminder', array ( 'json' => $request->get_params() ));
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode()); 

    return $response;
  }

  public function manually_approve_check(WP_REST_Request $request) {
    // get the ID check
    $client = $this->api();

    $res = $client->client->post('/api/checks/'.$request->get_params()['id'].'/manual-approval');
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response;
  }

  public function manually_reject_check(WP_REST_Request $request) {
    // get the ID check
    $client = $this->api();

    $res = $client->client->post('/api/checks/'.$request->get_params()['id'].'/manual-rejection');
    $data = json_decode((string) $res->getBody());
    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response;
  }

  public function associate_check_with_order(WP_REST_Request $request) {
    $client = $this->api();
    $res = $client->client->post('/api/webhooks/wc/associate-check-with-order', ['json' => $request->get_params()]);
    $data = json_decode((string) $res->getBody());

    if($res->getStatusCode() == 200) {
      $updated_order_meta = update_post_meta($order_id, 'real_id_check_id', $check_id);
    }

    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response; 
  }

  public function add_email_sender_signature(WP_REST_Request $request) {
    $client = $this->api();
    $res = $client->client->post('/api/shop/email-signature/add', ['json' => $request->get_params()]);
    $data = json_decode((string) $res->getBody());

    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response; 
  }

  public function check_email_sender_signature_verification_status(WP_REST_Request $request) {
    $client = $this->api();
    $res = $client->client->post('/api/shop/email-signature/check-verification-status', ['json' => $request->get_params()]);
    $data = json_decode((string) $res->getBody());

    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response; 
  }

  public function remove_email_sender_signature(WP_REST_Request $request) {
    $client = $this->api();
    $res = $client->client->post('/api/shop/email-signature/remove', ['json' => $request->get_params()]);
    $data = json_decode((string) $res->getBody());

    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response; 
  }

  public function reset_to_sandbox_mode(WP_REST_Request $request) {
    $sandboxLicenseKey = get_option('real_id_sandbox_license_key');
    update_option('real_id_license_key', $sandboxLicenseKey);
    update_option('real_id_sandbox_mode', true);
    $client = $this->api();
    // reset shop settings as well
    $reset_res = $client->client->post('/api/shop/license/reset-sandbox-mode', ['json' => [
      'sandboxLicenseKey' => get_option('real_id_sandbox_license_key'),
    ]]);

    if($reset_res->getStatusCode() !== 200) {
      $reset_res_data = json_decode((string) $reset_res->getBody());
      $response = new WP_REST_Response($reset_res_data);
      $response->set_status($reset_res->getStatusCode());
      return $response;
    }

    // get shop settings

    $res = $client->client->get('/api/shop');
    $data = json_decode((string) $res->getBody());

    $response = new WP_REST_Response($data);
    $response->set_status($res->getStatusCode());

    return $response; 
  }

  public function get_current_license_key(WP_REST_Request $request) {
    $license_key = real_id_get_license_key();

    $response = new WP_REST_Response([
      'license_key' => $license_key,
      'sandbox_mode' => real_id_in_sandbox_mode()
    ]);
    $response->set_status(200);

    return $response; 
  }
}
