<?php
namespace RealID;

use RealID\ApiClient;
use RealID\Models\IDCheck;
use RealID\CheckState;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Controller;
use Psr\Log\LoggerInterface;

class Webhooks extends WP_REST_Controller { 
  /**
   * Register the Webhook API routes with Wordpress
   * 
   * @return void
   */
  public function register_routes() {
    $version = '1';
    $namespace = 'real-id/v' . $version;

    register_rest_route( $namespace, '/webhooks/metadata', array(
        'methods' => 'POST',
        'callback' => array($this, 'update_metadata'),
        'permission_callback' => array($this, 'validate_webhook'),
        'args' => [
          'check_id' => [
            'description' => esc_html__('The check ID associated with the order & customer', 'realid'),
            'required' => false,
            'type' => 'string',
            'validate_callback' => function($value, $request, $key) {
              if(empty($value)) {
                return true;
              }

              return true;
            }
          ],
          'status' => [
            'description' => esc_html__('The new ID check status to update the order & customer meta tags to', 'realid'),
            'required' => true,
            'type' => 'string',
            'validate_callback' => function ($value, $request, $key) {
              return in_array($value, IDCheck::all_check_statuses());
            }
          ],
          'wc_order_id' => [
            'description' => esc_html__('The WooCommerce order to be updated with the new ID check status', 'realid'),
            'type' => 'int',
            'validate_callback' => function($param, $request, $key) {
              return is_numeric($param);
            },
            'sanitize_callback' => 'absint'
          ],
          'wc_customer_id' => [
            'description' => esc_html__('The WooCommerce customer to be updated with the new ID check status', 'realid'),
            'type' => 'int',
            'validate_callback' => function($param, $request, $key) {
              return is_numeric($param);
            },
            'sanitize_callback' => 'absint'
          ],
        ]
      ) 
    );

    register_rest_route( $namespace, '/webhooks/order/check/associate', array(
        'methods' => 'POST',
        'callback' => array($this, 'associate_resource_with_check'),
        'permission_callback' => array($this, 'validate_webhook'),
        'args' => [
          'check_id' => [
            'description' => esc_html__('The check ID associated with the order & customer', 'realid'),
            'required' => true,
            'type' => 'string',
            'validate_callback' => function($value, $request, $key) {
              return true;
            }
          ],
          'wc_order_id' => [
            'description' => esc_html__('The WooCommerce order to be associated with the new ID check status', 'realid'),
            'type' => 'int',
            'validate_callback' => function($param, $request, $key) {
              return is_numeric($param);
            },
            'sanitize_callback' => 'absint'
          ],
          'wc_customer_id' => [
            'description' => esc_html__('The WooCommerce customer to be associated with the new ID check status', 'realid'),
            'type' => 'int',
            'validate_callback' => function($param, $request, $key) {
              return is_numeric($param);
            },
            'sanitize_callback' => 'absint'
          ],
        ]
      ) 
    );

    register_rest_route( $namespace, '/metadata', array(
        'methods' => 'GET',
        'callback' => array($this, 'get_metadata'),
        'permission_callback' => array($this, 'validate_webhook'),
        'args' => [
          'wc_order_id' => [
            'description' => esc_html__('The WooCommerce order to be associated with the new ID check status', 'realid'),
            'type' => 'int',
            'validate_callback' => function($param, $request, $key) {
              return is_numeric($param);
            },
            'sanitize_callback' => 'absint'
          ],
          'wc_customer_id' => [
            'description' => esc_html__('The WooCommerce customer to be associated with the new ID check status', 'realid'),
            'type' => 'int',
            'validate_callback' => function($param, $request, $key) {
              return is_numeric($param);
            },
            'sanitize_callback' => 'absint'
          ],
        ]
      ) 
    );
  }

  /**
   * Update the given customer & order metadata to the current status
   * 
   * @NOTE there is a check state machine in place to keep errant unwanted status changing
   *        However, this can be ignored with force=true
   * @param WP_REST_REQUEST $request
   * @return WP_REST_RESPONSE
   */
  public function update_metadata(WP_REST_REQUEST $request) {
    $params = $request->get_params();

    // update woocommerce order & customer metadata & tags
    $check_id = $params['check_id'];
    $new_status = $params['status'];
    $wc_order_id = $params['wc_order_id'];
    $wc_customer_id = $params['wc_customer_id'];

    // attempt to set the metadata of the order
    //   1. Check that the order is present
    //   2. Make sure the new check status is qualified to change (for example: completed -> opened is not allowed) 
    if(isset($wc_order_id)) {
      $order = wc_get_order($wc_order_id);
      $current_status = $order->get_meta('real_id_check_status');
      $current_status = empty($current_status) ? 'delivered' : $current_status;

      if(!$order) {
        $response = new WP_REST_Response('Order not found.');
        $response->set_status(422);
      }

      if($current_status == $new_status) {
        $response = new WP_REST_Response([
          'message' => 'Same status, nothing to do here.',
          'current_status' => $current_status,
          'new_status' => $new_status
        ]);
        $response->set_status(200);

        return $response;
      }

      // if there's not status yet, it's safe to make one
      $checkState = new CheckState($current_status);

      // let's make sure the new state is allowed before we update the order & customer metadata
      $isAvailableTransition = isset($params['force']) && $params['force'] == 'true' ? true :  $checkState->stateMachine->getCurrentState()->can($new_status);

      if(!$isAvailableTransition) {
        // let the API know that this status change isn't possible.
        $response = new WP_REST_Response([
          'message' => 'Not a valid status transition',
          'current_status' => $current_status,
          'new_status' => $new_status
        ]);
        $response->set_status(422);

        return $response;
      }

      $order->update_meta_data('real_id_check_status', $new_status);
      $order->save();
    }

    if(isset($wc_customer_id)) {
      // attempt to set the metadata of the customer
      update_user_meta($wc_customer_id, 'real_id_check_status', $new_status);
    }

    $response = new WP_REST_Response([ 'message' => 'Saved status update', 'new_status' => $new_status, 'prior_status' => $current_status]);
    $response->set_status(200);
    return $response;
  }

  /**
   * For remote troubleshooting to make sure metadata is accurate 
   */
  public function get_metadata(WP_REST_Request $request) {
    $params = $request->get_params();
    $data = [];

    if(isset($params['wc_customer_email'])) {
      $customer_by_email = get_user_by('email', $params['wc_customer_email']);

      if(!$customer_by_email) {
        $response = new WP_REST_Response([
          'message' => 'customer by that email not found',
          'email' => $params['wc_customer_email'],
        ]);
        $response->set_status(404);

        return $response; 
      }

      $data['wc_customer_email'] = $params['wc_customer_email'];
      $data['wc_customer_id'] = $customer_by_email->ID;
      $data['wc_customer_status'] = get_user_meta($customer_by_email->ID, 'real_id_check_status', true);
      $data['wc_customer_check_id'] = get_user_meta($customer_by_email->ID, 'real_id_check_id', true);
    }

    if(isset($params['wc_customer_id'])) {
      $data['wc_customer_status'] = get_user_meta($params['wc_customer_id'], 'real_id_check_status', true);
    }

    if(isset($params['wc_order_id'])) {
      $data['wc_order_status'] = get_post_meta($params['wc_order_id'], 'real_id_check_status', true);
    }

    if(isset($params['wc_order_id'])) {
      $data['wc_order_check_id'] = get_post_meta($params['wc_order_id'], 'real_id_check_id', true);
    }

    if(isset($params['wc_customer_id'])) {
      $data['wc_customer_check_id'] = get_user_meta($params['wc_user_id'], 'real_id_check_id', true);
    }

    $response = new WP_REST_Response($data);
    $response->set_status(200);

    return $response;
  }

  /**
   * Associate the given order or customer with the given check ID
   * 
   * This will set the order and/or customer's real_id_check_id meta tag
   */
  public function associate_resource_with_check(WP_REST_REQUEST $request) {
    $response = new WP_REST_Response();
    $params = $request->get_params();
    // by default the endpoint is not able to update the customer or order meta without valid params
    $updated_order_meta = false;
    $updated_customer_meta = false;

    // update woocommerce order & customer metadata & tags
    $wc_order_id = $params['wc_order_id'];
    $wc_customer_id = $params['wc_customer_id'];
    $check_id = $params['check_id'];

    if((!$wc_order_id || !$wc_customer_id) && !$check_id) {
      // we need both the check ID and a taggable resource like a customer or order to perform the association
      $response->set_data([ 'message' => 'Requires a wc_customer_id or wc_order_id and a check_id' ]);
      $response->set_status(422);
      return $response;
    }

    if(isset($wc_order_id)) {
      // attempt to set the metadata of the order
      $order = wc_get_order($wc_order_id);
      if(!$order) {
        $response->set_status(422);
        $response->set_data([ 'message' => 'Order not found.' ]);
        return $response;
      }
      $updated_order_meta = update_post_meta($wc_order_id, 'real_id_check_id', $check_id);
    }

    if(isset($wc_customer_id)) {
      // attempt to set the metadata of the customer
      $updated_customer_meta = update_user_meta($wc_customer_id, 'real_id_check_id', $check_id);
    }

    $response->set_status(200);
    $response->set_data([
      'message' => sprintf('Associated check %s. Note, a false does not necessarily mean the update failed or resource key is incorrect. It only returns true when an actual update was needed: https://developer.wordpress.org/reference/functions/update_post_meta/', $params['check_id']),
      'associated_order_successfully' => $updated_order_meta,
      'associated_customer_successfully' => $updated_customer_meta,
    ]);
    return $response;
  }

  /**
   * Validate the webhook's authenticity by confirming it is using a valid license key
   */
  public function validate_webhook(WP_REST_Request $request) {
    $bearer = $request->get_header('Authorization');

    if(!isset($bearer)) {
      return false;
    }

    $token_parts = explode('Bearer ', $bearer);
    if(count($token_parts) != 2) {
      return false;
    }

    $token = $token_parts[1];
    $licenseKey = real_id_get_license_key();

    return $token == $licenseKey;
  }
}
