<?php
/**
 * Plugin Name: Identity Verification for WooCommerce
 * Plugin URI: https://getverdict.com/real-id
 * Description: Verify your customers' real identities – instantly and securely.
 * Author: verdictapps
 * Author URI: https://getverdict.com/
 * Version: 1.2.2
 * Developer: Verdict Apps
 * Developer URI: https://getverdict.com/
 * Text Domain: woocommerce-extension
 * Domain Path: /languages
 * Tested up to: 5.8
 *
 * Woo: 12345:342928dfsfhsf8429842374wdf4234sfd
 * WC requires at least: 3.0
 * WC tested up to: 3.0
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

namespace RealID;

use RealID\Api;
use RealID\Webhooks;
use RealID\WcRealID;
use RealID\ApiClient;

if ( ! class_exists( 'WC_Integration_Real_ID' ) ) :

require(plugin_dir_path(__FILE__) . '/vendor/autoload.php');

class WC_Integration_Real_ID {

	/**
	* Construct the plugin.
	*/
	public function __construct() {
		add_action('plugins_loaded', array( $this, 'init' ) );
    add_filter('plugin_action_links', array($this, 'plugin_action_links'), 10, 2);
    add_action('admin_enqueue_scripts', [$this, 'register_admin_bundle_js']);
    add_action('wp_enqueue_scripts', [$this, 'register_shop_scripts']);
    add_action('wp_enqueue_scripts', [$this, 'register_idv_flow_scripts']);
    add_action('rest_api_init', array( $this, 'register_api_routes' ));
    add_action('init', [$this, 'rewrite_admin_url_rules']);
    add_filter('script_loader_tag', [$this, 'add_script_module_type_attribute'] , 10, 3);
    add_shortcode('real_id_user_verification_status', [$this, 'real_id_user_verification_status_shortcode']);
    
    if (getenv('GITPOD_WORKSPACE_URL')) {
      define('REAL_ID_API_HOST', getenv('GITPOD_WORKSPACE_URL'));
      define('REAL_ID_DEV', true);
    }
	}

  function rewrite_admin_url_rules() {
    add_rewrite_rule('^real-id/(.+)?', 'index.php?pagename=/wp-admin/admin', 'top');
  }

  public function register_api_routes() {
    $api = real_id_app()->get(Api::class);
    $api->register_routes();
    $webhooks = real_id_app()->get(Webhooks::class);
    $webhooks->register_routes();
  }

  /**
   * Set up the current shop with a test key if none stored
   */
  public function setup_sandbox_key() {
    $sandboxLicenseKey = get_option('real_id_sandbox_license_key');
    $licenseKey = get_option('real_id_license_key');

    if(empty($sandboxLicenseKey) && empty($licenseKey)) {
      $client = new ApiClient();
      $res = $client->client->post('/api/shop/wc/install', [ 'json' => [
        'url' => home_url(),
      ]]);

      $data = json_decode((string) $res->getBody());

      if($res->getStatusCode() == 200) {
        add_option('real_id_sandbox_license_key', $data->shop->licenseKey);
        // if the sandbox mode doesn't exist enable add it
        // also make sure the shop isn't already in live mode with good license key
        if(!real_id_option_exists('real_id_sandbox_mode') && !real_id_option_exists('real_id_license_key')) {
          add_option('real_id_sandbox_mode', true);
        }
      }
    }
  }

	/**
	* Initialize the plugin.
	*/
	public function init() {
    // create a local sandbox key if none stored locally
    $this->setup_sandbox_key();

		// Checks if WooCommerce is installed.
		if ( class_exists( 'WC_Integration' ) ) {
      // including the class once by file instead of autoloader,
      // this way PHP has context of the file OUTSIDE of our namespaced plugin execution
      // this is necessary for the upstream WooCommerce plugin to find our integration file
      include_once(plugin_dir_path(__FILE__) . 'includes/WCRealID.php');
			// Register the integration.
			add_filter( 'woocommerce_integrations', array( $this, 'add_integration' ) );
		}
	}

  public function plugin_action_links($links, $file) {
    if (plugin_basename(__FILE__) !== $file) {
      return $links;
    }

    $settings_link = '<a href="admin.php?page=real-id">Settings</a>';

    array_unshift($links, $settings_link);

    return $links;
  }

	/**
	 * Add a new integration to WooCommerce.
	 */
	public function add_integration( $integrations ) {
    $integrations[] = WCRealID::class;
		return $integrations;
	}

  /**
   * Register the admin React javascript bundle with wordpress so we can find the development server
   */
  public function register_admin_bundle_js($hook) {
    if (defined('REAL_ID_DEV') ) {
      // DEV React dynamic loading
      $parts = explode('https://', getenv('GITPOD_WORKSPACE_URL'));
      $js_to_load = sprintf('https://3005-%s/bundle.js', $parts[1]);
    } else {
      $js_to_load = plugin_dir_url( __FILE__ ) . 'dist/bundle.js';
      $css_to_load = plugin_dir_url( __FILE__ ) . 'dist/styles.css';
      wp_enqueue_style('real_id_css', $css_to_load);
    }

    wp_enqueue_script('real_id_js', $js_to_load, '', mt_rand(10,1000), true);

    // share the nonce with the frontend which is required for authenticating JS requests
    wp_add_inline_script(
      'real_id_js',
      'var realIdApiSettings = ' .
        json_encode(array(
          'root' => esc_url_raw(rest_url()),
          'nonce' => wp_create_nonce('wp_rest')
        )),
      'before'
    );

    $asset_url = plugin_dir_url(__FILE__) . 'public/assets';

    // share the plugin url with the frontend too
    wp_add_inline_script(
      'real_id_js',
      sprintf('var realIdAssetsUrl = "%s"',  $asset_url),
      'before'
    );
  }

  public function register_shop_scripts($hook) {
    wp_enqueue_script('real_id_shop_js', plugin_dir_url( __FILE__ ) .'/js/shop.js', array(), mt_rand(10, 10000), true);

    // share the home url with the frontend too
    wp_add_inline_script(
      'real_id_shop_js',
      sprintf('var realIdShopName= "%s"',  real_id_get_shop_name()),
      'before'
    );
  }

  /**
   * Register the location of the real-id-flow frontend script
   */
  public function register_idv_flow_scripts($hook) {

    $delivery_methods = get_option('real_id_delivery_methods');

    // If there is no special delivery methods set yet, then don't register the frontend script on the page
    // if(!$delivery_methods || is_null($delivery_methods) || !in_array('preCheckout', $delivery_methods)) {
    //   return;
    // }

    if (defined('REAL_ID_DEV')) {
      // TODO: figure out webpack hot reloading websocket
      $parts = explode('https://', getenv('GITPOD_WORKSPACE_URL'));
      $js_to_load = sprintf('https://3006-%s/bundle.js', $parts[1]);
    } elseif (defined('REAL_ID_STAGING')) {
      $js_to_load = 'https://real-id-flow-staging.getverdict.com/assets/index.js';
      $css_to_load = 'https://real-id-flow-staging.getverdict.com/assets/index.css';
      wp_enqueue_style('real_id_flow_css', $css_to_load);
    } else {
      $js_to_load = 'https://real-id-flow.getverdict.com/assets/index.js';
      $css_to_load = 'https://real-id-flow.getverdict.com/assets/index.css';
      wp_enqueue_style('real_id_flow_css', $css_to_load);
    }

    wp_enqueue_script('real_id_flow_js', $js_to_load, '', mt_rand(10,1000), true);
  }

  /**
   * Real ID Flow is exported as a module script
   *    Altering the script tag for this specific tag so we can load it appropriately
   * 
   */
  public function add_script_module_type_attribute($tag, $handle, $src) {
      // if not your script, do nothing and return original $tag
      if ( 'real_id_flow_js' !== $handle ) {
          return $tag;
      }
      // change the script tag by adding type="module" and return it.
      $tag = '<script type="module" src="' . esc_url( $src ) . '"></script>';
      return $tag;
  }

  /**
   * Real ID shortcode for user verification status
   */
  public function real_id_user_verification_status_shortcode($atts = [], $content = null) {
    $user_id = $atts['user_id'];
    $status = get_user_meta($user_id, 'real_id_check_status', true);

    if($status == 'completed') {
      return '<span class="real-id-verified"><span class="dashicons dashicons-yes-alt"></span>&nbsp;Verified</span>';
    } else {
      return '<span class="real-id-unverified"><span class="dashicons dashicons-dismiss"></span>&nbsp;Unverified</span>';
    }

    return $content;
  }

}

$WC_Integration_Real_ID = new WC_Integration_Real_ID( __FILE__ );

endif;
