import React, { useContext } from "react";
import { Flex, Box } from "rebass";
import {
  Card,
  Layout,
  TextContainer,
  ChoiceList,
  DataTable,
  Subheading,
  TextStyle,
  FormLayout,
  Checkbox,
  TextField,
  Banner,
  Link,
} from "@shopify/polaris";
import CrossCheckAddressIcon from "assets/cross_check_shipping_address.svg";
import CrossCheckLastNameIcon from "assets/cross_check_last_name.svg";
import IdCheckOption from "components/IdCheckOption";
import { AiOutlineIdcard } from "react-icons/ai";
import { BiFace } from "react-icons/bi";
import SignatureRequired from "components/checks/SignatureRequired";
import { Redirect } from "@shopify/app-bridge/actions";
import { Context } from "@shopify/app-bridge-react";
import SubscriptionGateway from "components/auth/SubscriptionGateway";

export default function AdditionalCheckRequirements({
  values,
  setFieldValue,
  shop,
  errors,
}) {
  const shopifyApp = useContext(Context);
  const redirect = Redirect.create(shopifyApp);

  return (
    <Layout.AnnotatedSection
      title="Additional ID Check Requirements"
      description={
        <>
          <p>Require additional personal information during the ID check.</p>
        </>
      }
    >
      <Card fullWidth={true}>
        <SubscriptionGateway permission="idv_check_type" minimumPlan="Plus">
          <Card.Section title="ID Photo Requirements">
            <Flex>
              <IdCheckOption
                selected={values.idCheckType === "id"}
                onClick={() => {
                  setFieldValue("idCheckType", "id");
                }}
                title="ID Only"
                icons={<AiOutlineIdcard />}
                subtitle="More convenient"
                helperText="Only require a photo of the customer's valid ID. The ID will be scanned for authenticity."
              />
              <IdCheckOption
                selected={values.idCheckType === "idv"}
                onClick={() => {
                  setFieldValue("idCheckType", "idv");
                }}
                icons={
                  <Flex alignItems="center" justifyContent="center" width={0.5}>
                    <AiOutlineIdcard />
                    <TextStyle variation="subdued">+</TextStyle>
                    <BiFace />
                  </Flex>
                }
                title="ID and Headshot"
                subtitle="More secure"
                helperText="The face in the headshot will be cross referenced with the face photo on the ID document. Also includes the ID authenticity scan."
              />
            </Flex>
          </Card.Section>
        </SubscriptionGateway>
        <SubscriptionGateway permission="age_requirements" minimumPlan="Plus">
          <Card.Section title="Age Requirements">
            <Box my={2}>
              <TextContainer spacing="tight">
                <TextStyle variation="subdued">
                  If your products are age restricted, you can have Real ID
                  enforce a minimum age to pass ID verification.
                </TextStyle>
                <p>
                  <TextStyle variation="subdued">
                    You'll be alerted if a customer is underage.
                  </TextStyle>
                </p>
              </TextContainer>
            </Box>
            {shop.subscriptionPlan === "lite" && (
              <Banner status="warning">
                <TextContainer spacing="loose">
                  <p>
                    Automatic ID to order consistency checks are only available
                    with the A.I. plan.
                  </p>
                  <p>
                    <Link
                      onClick={() => {
                        redirect.dispatch(Redirect.Action.APP, "/plans-new");
                      }}
                    >
                      You can change your plan at any time.
                    </Link>
                  </p>
                </TextContainer>
              </Banner>
            )}
            <ChoiceList
              title="Minimum age to pass an ID check"
              choices={[
                {
                  label: "No minimum age requirement",
                  value: "disabled",
                },
                {
                  label: "18",
                  value: "18",
                },
                {
                  label: "21",
                  value: "21",
                },
                {
                  label: "Other age",
                  value: "other",
                  renderChildren: (isSelected) =>
                    isSelected && (
                      <TextField
                        type="number"
                        error={errors.minimumAge}
                        suffix="years old"
                        value={
                          values.minimumAge == "other"
                            ? ""
                            : values.minimumAge + ""
                        }
                        onChange={(value) => {
                          setFieldValue("minimumAge", value);
                        }}
                      />
                    ),
                },
                {
                  label: "Canadian and U.S. Legal Smoking Ages",
                  value: "legal_canadian_smoking_ages",
                  renderChildren: (isSelected) =>
                    isSelected && (
                      <Box my={2}>
                        <Box my={2}>
                          <p>
                            Below are the age minimums that will be enforced for
                            each Canadian province and the U.S.
                          </p>
                        </Box>
                        <Subheading element="h2">
                          Canadian Tobacco Purchase Age Restrictions
                        </Subheading>
                        <DataTable
                          columnContentTypes={["text", "numeric"]}
                          headings={["Province", "Minimum Age"]}
                          rows={[
                            ["Alberta", "18+"],
                            ["British Columbia", "19+"],
                            ["Manitoba", "18+"],
                            ["Ontario", "19+"],
                            ["Prince Edward Island", "21+"],
                            ["Quebec", "18+"],
                            ["Saskatchewan", "18+"],
                            ["New Brunswick", "19+"],
                            ["Newfoundland and Labrador", "19+"],
                            ["Northwest Territories", "18+"],
                            ["Nova Scotia", "19+"],
                            ["Nunavut", "18+"],
                            ["Yukon Territories", "18+"],
                          ]}
                        />
                        <Box my={2}>
                          <Subheading element="h2">
                            United States Tobacco Purchase Age Restrictions
                          </Subheading>
                          <DataTable
                            columnContentTypes={["text", "numeric"]}
                            headings={["State", "Minimum Age"]}
                            rows={[["All States", "21+"], ,]}
                          />
                        </Box>
                        <TextStyle variation="subdued">
                          All other countries will not be checked for an age
                          minimum.
                        </TextStyle>
                      </Box>
                    ),
                },
              ]}
              disabled={shop.subscriptionPlan === "lite"}
              // const ageRequirementSelection = useMemo(() => {
              //   if (!minimumAgeEnabled) {
              //     return "disabled";
              //   }

              //   if ([18, 21].includes(parseInt(minimumAge))) {
              //     // convert to a string so the textfield works proper
              //     return [minimumAge + ""];
              //   }

              //   if (minimumAge === "legal_canadian_smoking_ages") {
              //     return "legal_canadian_smoking_ages";
              //   }

              //   return "other";
              // }, [minimumAge, minimumAgeEnabled]);
              selected={
                shop.subscriptionPlan === "lite"
                  ? "disabled"
                  : (() => {
                      if (!values.minimumAgeEnabled) {
                        return "disabled";
                      }
                      if ([18, 21].includes(parseInt(values.minimumAge))) {
                        // convert to a string so the textfield works proper
                        return [values.minimumAge + ""];
                      }
                      if (values.minimumAge === "legal_canadian_smoking_ages") {
                        return "legal_canadian_smoking_ages";
                      }
                      return "other";
                    })() //ageRequirementSelection
              }
              onChange={([choice]) => {
                if (choice == "disabled") {
                  setFieldValue("minimumAgeEnabled", false);
                  setFieldValue("minimumAge", "disabled");
                  return;
                }

                // if (choice == "other") {
                //   setFieldValue("minimumAge", "other");
                //   // don't `return` here because we need to set the minimumAgeEnabled to true
                // }

                // if (choice == "legal_canadian_smoking_ages") {
                //   setFieldValue(
                //     "minimumAge",
                //     "legal_canadian_smoking_ages"
                //   );
                //   // don't `return` here because we need to set the minimumAgeEnabled to true
                // }

                setFieldValue("minimumAge", choice.toString());
                setFieldValue("minimumAgeEnabled", true);
              }}
            />
          </Card.Section>
        </SubscriptionGateway>
        <SubscriptionGateway
          permission="id_to_order_consistency_checks"
          minimumPlan="Plus"
        >
          <Card.Section title="ID to Order consistency checks">
            <Box my={3}>
              <TextContainer spacing="loose">
                <TextStyle variation="subdued">
                  Automatically reject IDs for orders that have mismatching
                  details. This will help reduce fraud by ensuring the
                  customer's supplied ID matches the personal information on the
                  order.
                </TextStyle>
              </TextContainer>
              {shop.subscriptionPlan === "lite" && (
                <Banner status="warning">
                  <TextContainer spacing="loose">
                    <p>
                      Automatic ID to order consistency checks are only
                      available with the A.I. plan.
                    </p>
                    <p>
                      <Link
                        onClick={() => {
                          redirect.dispatch(Redirect.Action.APP, "/plans-new");
                        }}
                      >
                        You can change your plan at any time.
                      </Link>
                    </p>
                  </TextContainer>
                </Banner>
              )}
              <FormLayout>
                <Flex mt={4} justifyContent="center">
                  <Box width={0.8}>
                    <CrossCheckLastNameIcon />
                  </Box>
                </Flex>
                <Checkbox
                  label={
                    <>
                      Require the <strong>name on the ID</strong> match the{" "}
                      <strong>customer's name</strong> on the{" "}
                      <strong>billing address</strong>
                    </>
                  }
                  helpText={
                    <TextContainer>
                      <p>
                        Only accept ID documents that match the full name on the
                        order's billing address.
                      </p>
                      <p>
                        This also includes fuzzy matching the names in cases
                        such as matching nicknames or shortened names. Real ID
                        will attempt to match John to Johnathon, or Jenn to
                        Jennifer, etc.
                      </p>
                    </TextContainer>
                  }
                  disabled={shop.subscriptionPlan === "lite"}
                  checked={
                    shop.subscriptionPlan === "lite"
                      ? false
                      : values.crossCheckCustomerLastName
                  }
                  onChange={(checked) => {
                    setFieldValue("crossCheckCustomerLastName", checked);
                  }}
                />
                <Flex mt={5} justifyContent="center">
                  <Box width={0.8}>
                    <CrossCheckAddressIcon />
                  </Box>
                </Flex>
                <Checkbox
                  label={
                    <>
                      Require the <strong>address on the ID</strong> match the{" "}
                      <strong>order's shipping address</strong>
                    </>
                  }
                  helpText={
                    <TextContainer>
                      <p>
                        Only accept ID documents that match the order's shipping
                        address.{" "}
                      </p>
                      <p>
                        Some documents such as passports only include the state
                        & country fields. We'll use the most specific details
                        available on the ID document & shipping address to make
                        a match.
                      </p>
                    </TextContainer>
                  }
                  disabled={shop.subscriptionPlan === "lite"}
                  checked={
                    shop.subscriptionPlan === "lite"
                      ? false
                      : values.crossCheckShippingAddress
                  }
                  onChange={(checked) => {
                    setFieldValue("crossCheckShippingAddress", checked);
                  }}
                />
              </FormLayout>
            </Box>
          </Card.Section>
        </SubscriptionGateway>
        <SubscriptionGateway permission="e_signatures" minimumPlan="Lite">
          <Card.Section>
            <SignatureRequired
              shop={shop}
              label="Require the customer signature to complete an ID check"
              checked={values.signatureRequired}
              setChecked={(checked) => {
                setFieldValue("signatureRequired", checked);
              }}
            />
          </Card.Section>
        </SubscriptionGateway>
      </Card>
    </Layout.AnnotatedSection>
  );
}
