import React, { useState } from "react";
import {
  Card,
  TextField,
  FormLayout,
  Banner,
  TextContainer,
  TextStyle,
  Layout,
} from "@shopify/polaris";
import get from "lodash/get";
import isNil from "lodash/isNil";
import { useToast } from "providers/ToastProvider";
import { Box } from "rebass";
import SubscriptionGateway from "components/auth/SubscriptionGateway";
import isEmail from "validator/lib/isEmail";
import {
  addEmailSignature,
  checkEmailSignatureVerificationStatus,
  removeEmailSignature,
} from "services/wpApi";

export default function EmailSenderSignatureSettings({ shop, setShop }) {
  const setToast = useToast();
  const [fromEmail, setFromEmail] = useState(
    get(shop, "emailSenderSignature.EmailAddress", shop.contactEmail)
  );
  const [fromName, setFromName] = useState(
    get(shop, "emailSenderSignature.Name", "Support")
  );
  let primaryFooterAction = {
    content: "Verify",
    disabled: !fromName || !fromEmail || !isEmail(fromEmail),
    onAction: () => {
      // verify domain
      addEmailSignature({
        fromEmail,
        fromName,
      })
        .then(({ data }) => {
          setShop(data.shop);
          console.log(data);
        })
        .catch((e) => {
          console.error(e);
          setToast(
            `Unable to add the email ${fromEmail} for email verification. Please contact support.`
          );
        });
    },
  };
  let secondaryFooterActions = [];

  if (shop?.emailSenderSignature) {
    primaryFooterAction = {
      content: "Check Verification Status",
      onAction: () => {
        // verify domain
        checkEmailSignatureVerificationStatus()
          .then(({ data }) => {
            setShop(data.shop);
            console.log(data);
          })
          .catch((e) => {
            console.error(e);
            setToast(
              `Email not yet verified, please check your inbox for an email from support@postmarkapp.com`
            );
          });
      },
    };
  }

  if (shop?.emailSenderSignature?.Confirmed) {
    primaryFooterAction = false;
  }

  if (shop.emailSenderSignature) {
    secondaryFooterActions = [
      {
        destructive: true,
        content: "Remove Email Address",
        onAction: () => {
          removeEmailSignature()
            .then(({ data }) => {
              setToast(`Removed your email address.`);
              setShop(data.shop);
              console.log(data);
            })
            .catch((e) => {
              console.error(e);
              setToast(
                `Unable to remove the email ${fromEmail} from settings. Please contact support.`
              );
            });
        },
      },
    ];
  }

  return (
    <Layout.AnnotatedSection
      title="Send ID checks with your own email address"
      description={
        <>
          <p>
            Build trust with your customers by using your email address to send
            ID checks.
          </p>
          <br />
          <p>Verify your email below to get started.</p>
        </>
      }
    >
      {/* TODO: make a specific permission for this, but make a task to apply it to old accounts too */}
      <SubscriptionGateway permission="branding" minimumPlan="Pro">
        <Card
          fullWidth
          primaryFooterAction={primaryFooterAction}
          secondaryFooterActions={secondaryFooterActions}
          title="Send ID verification notifications to your customers from your email address"
        >
          <Card.Section>
            {shop?.emailSenderSignature?.Confirmed && (
              <Banner
                status="success"
                title={
                  <>
                    Now sending ID checks and notifications to customers with{" "}
                    <strong>{shop.emailSenderSignature.EmailAddress}</strong>
                  </>
                }
              >
                You've verifed ownership of{" "}
                {shop.emailSenderSignature.EmailAddress}! ID check notifications
                to customers will now be sent from this email.
              </Banner>
            )}
            {!shop?.emailSenderSignature?.Confirmed &&
              shop?.emailSenderSignature && (
                <Banner status="info" title="Just one more step">
                  <TextContainer>
                    <p>
                      Check your inbox at{" "}
                      <code>{shop?.emailSenderSignature?.EmailAddress}</code>{" "}
                      for an email from Postmark to finish email verification.
                    </p>
                    <p>
                      This email will contain a link to verify your email
                      address.
                    </p>
                  </TextContainer>
                </Banner>
              )}
            {!shop?.emailSenderSignature && (
              <TextContainer>
                <p>
                  <TextStyle variation="subdued">
                    To build customer trust, you can use your own{" "}
                    <strong>email address</strong> to send ID verification
                    checks.
                  </TextStyle>
                </p>
                <p>
                  <TextStyle variation="subdued">
                    ID check notifications will be sent to the customer using
                    this email address:
                    <ul>
                      <li>ID check required</li>
                      <li>ID check passed</li>
                      <li>ID check in review</li>
                    </ul>
                  </TextStyle>
                </p>
                <p>
                  <TextStyle variation="subdued">
                    Use your customer support email address below:
                  </TextStyle>
                </p>
              </TextContainer>
            )}
            <Box my={3}>
              <FormLayout>
                <FormLayout.Group>
                  <TextField
                    label="Name"
                    value={fromName}
                    disabled={!isNil(shop.emailSenderSignature)}
                    onChange={(value) => {
                      setFromName(value);
                    }}
                  />
                  {get(shop, "settings.emailSenderVerificationStatus") !==
                    "verified" && (
                    <TextField
                      type="email"
                      label="Email Address"
                      disabled={!isNil(shop.emailSenderSignature)}
                      value={fromEmail}
                      onChange={(value) => {
                        setFromEmail(value);
                      }}
                    />
                  )}
                </FormLayout.Group>
              </FormLayout>
            </Box>
          </Card.Section>
        </Card>
      </SubscriptionGateway>
    </Layout.AnnotatedSection>
  );
}
