import React, { useEffect, useState, useMemo, useContext } from "react";
import { Box, Flex } from "rebass";
import {
  Tabs,
  Card,
  ResourceItem,
  TextStyle,
  Badge,
  ResourceList,
  Filters,
  Icon,
} from "@shopify/polaris";
import Toggle from "react-toggle";
import { SessionContext } from "components/contexts/SessionContext";
import useApi from "components/contexts/useApi";
import { useFormikContext } from "formik";
import useDebounce from "components/hooks/useDebounce";
import get from "lodash/get";
import uniq from "lodash/uniq";
import clone from "lodash/clone";
import has from "lodash/has";
import useToast from "components/hooks/useToast";
import useGraphqlClient from "components/contexts/useGraphql";
import { gql } from "@apollo/client";
import { CollectionsMajor } from "@shopify/polaris-icons";
import * as Sentry from "@sentry/nextjs";

export default function RequiredIdProductsAndCollections({}) {
  const setToast = useToast();
  const app = useContext(SessionContext);
  const api = useApi(app);
  const graphqlClient = useGraphqlClient(app);
  const { values, setFieldValue } = useFormikContext();
  const [products, setProducts] = useState([]);
  const [productQuery, setProductQuery] = useState("");
  const debouncedProductQuery = useDebounce(productQuery, 500);

  const [collections, setCollections] = useState([]);
  const [collectionQuery, setCollectionQuery] = useState("");
  const debouncedCollectionQuery = useDebounce(collectionQuery, 500);

  const [productsOrCollectionsNav, setProductsOrCollectionsNav] = useState(0);

  async function hydrateRequiredIdProducts(requiredIdProducts) {
    graphqlClient
      .query({
        query: gql`
          query getProducts($ids: [ID!]!) {
            nodes(ids: $ids) {
              ... on Product {
                id
                title
                createdAt
                images(first: 1) {
                  edges {
                    node {
                      originalSrc
                      transformedSrc(maxHeight: 100, maxWidth: 100)
                    }
                  }
                }
              }
            }
          }
        `,
        variables: { ids: requiredIdProducts },
      })
      .then(({ data }) => {
        setProducts(data.nodes.map(clone));
      })
      .catch((err) => {
        Sentry.captureException(err);
        console.log(err);
      });
  }

  async function hydrateRequiredIdCollections(requiredIdCollections) {
    graphqlClient
      .query({
        query: gql`
          query getCollections($ids: [ID!]!) {
            nodes(ids: $ids) {
              ... on Collection {
                id
                title
                handle
                productsCount
                image {
                  originalSrc
                  transformedSrc(maxHeight: 100, maxWidth: 100)
                }
              }
            }
          }
        `,
        variables: { ids: requiredIdCollections },
      })
      .then(({ data }) => {
        setCollections(data.nodes.map(clone));
      })
      .catch((err) => {
        console.log(err);
      });
  }

  // initial load, hydrate the required ID products with the photos and details from the GraphQL API
  useEffect(() => {
    // if there are blocked products, populate the resource list now
    if (
      values.requiredIdProducts.length > 0 &&
      has(values.requiredIdProducts[0], "isIdRequired")
    ) {
      hydrateRequiredIdProducts(values.requiredIdProducts);
    }

    // if there are blocked products, populate the resource list now
    if (
      values.requiredIdCollections.length > 0 &&
      has(values.requiredIdCollections[0], "isIdRequired")
    ) {
      hydrateRequiredIdCollections(values.requiredIdCollections);
    }
  }, [values.requiredIdCollections, values.requiredIdProducts]);

  // search for products by name
  useEffect(() => {
    if (!productQuery) {
      hydrateRequiredIdProducts(values.requiredIdProducts);
      return;
    }

    api
      .get("/api/shop/products", { params: { productQuery } })
      .then(({ data }) => {
        setProducts(data.map((p) => p.node));
      })
      .catch((err) => {
        console.log(err);
        setToast(
          "Sorry there was a problem retrieving your store's products. Please contact support."
        );
      });
  }, [debouncedProductQuery]);

  // search for collections by name
  useEffect(() => {
    if (!collectionQuery) {
      hydrateRequiredIdCollections(values.requiredIdCollections);
      return;
    }

    api
      .get("/api/shop/collections", { params: { collectionQuery } })
      .then(({ data }) => {
        setCollections(data.map((p) => p.node));
      })
      .catch((err) => {
        console.log(err);
        setToast(
          "Sorry there was a problem retrieving your store's collections. Please contact support."
        );
      });
  }, [debouncedCollectionQuery]);

  // responsible for keeping track of which product are or aren't on the list
  const productsMemo = useMemo(() => {
    console.log(products);
    return products
      .filter((p) => p != null)
      .map((product) => {
        product.isIdRequired = values.requiredIdProducts.includes(product.id);
        return product;
      });
  }, [products, values.requiredIdProducts]);

  // responsible for keeping track of which collections are or aren't on the list
  const collectionsMemo = useMemo(() => {
    return collections
      .filter((c) => c != null)
      .map((collection) => {
        collection.isIdRequired = values.requiredIdCollections.includes(
          collection.id
        );
        return collection;
      });
  }, [collections, values.requiredIdCollections]);

  return (
    <Box my={3}>
      <Tabs
        tabs={[
          {
            id: "products",
            content: (
              <span>
                <Badge status="new">{values.requiredIdProducts.length}</Badge>
                Products
              </span>
            ),
            accessibilityLabel: "ID required products",
            panelID: "products-content",
          },
          {
            id: "collections",
            content: (
              <span>
                <Badge status="new">
                  {values.requiredIdCollections.length}
                </Badge>
                Collections
              </span>
            ),
            accessibilityLabel: "ID required collections",
            panelID: "collections-content",
          },
        ]}
        selected={productsOrCollectionsNav}
        onSelect={(selection) => setProductsOrCollectionsNav(selection)}
        fitted
      >
        {productsOrCollectionsNav === 0 ? (
          <Card.Section title="Products Requiring ID Verification">
            <TextStyle variation="subdued">
              Choose which products require ID verification. An ID verification
              link will be sent on checkout if an order contains one or more of
              the products below.
            </TextStyle>
            <Flex alignItems="center" my={2} mx={3}>
              <Toggle
                checked={values.requiredIdProductsEnabled}
                onChange={(e) => {
                  setFieldValue("requiredIdProductsEnabled", e.target.checked);
                }}
              />
              &nbsp;
              {values.requiredIdProductsEnabled ? (
                <>
                  Products requiring ID verification is&nbsp;
                  <strong>active</strong>
                </>
              ) : (
                <>
                  Products requiring ID verification is&nbsp;
                  <strong>disabled</strong>
                </>
              )}
            </Flex>

            {values.requiredIdProductsEnabled && (
              <ResourceList
                filterControl={
                  <Filters
                    filters={[]}
                    appliedFilters={[]}
                    onClearAll={() => {}}
                    queryValue={productQuery}
                    onQueryChange={setProductQuery}
                    onQueryClear={() => {
                      setProductQuery("");
                    }}
                    totalItemsCount={products.length}
                  />
                }
                renderItem={(product) => {
                  const imageUrl =
                    get(product, "images.edges").length > 0
                      ? get(product, "images.edges")[0].node.transformedSrc
                      : "";

                  return (
                    <ResourceItem
                      media={<img src={imageUrl} />}
                      onClick={() => {
                        // if product is already on the list, remove it
                        if (values.requiredIdProducts.includes(product.id)) {
                          const newRequiredIdProducts =
                            values.requiredIdProducts.filter(
                              (p) => p != product.id
                            );
                          setFieldValue(
                            "requiredIdProducts",
                            newRequiredIdProducts
                          );
                          setProductQuery("");
                        } else {
                          // if it is not on the list, add it
                          const newRequiredIdProducts = uniq([
                            product.id,
                            ...values.requiredIdProducts,
                          ]);

                          setFieldValue(
                            "requiredIdProducts",
                            newRequiredIdProducts
                          );
                          // setProductQuery("");
                        }
                      }}
                    >
                      {product.title}
                      {product.isIdRequired && (
                        <Box my={3}>
                          <Badge status="success">
                            ID Verification Required
                          </Badge>
                        </Box>
                      )}
                    </ResourceItem>
                  );
                }}
                items={productsMemo}
              />
            )}

            {/* <Pagination 
                              hasPrevious
                              onPrevious={() => {

                              }}
                              hasNext
                              onNext={() => {

                              }}
                            /> */}
          </Card.Section>
        ) : (
          <Card.Section title="Collections Requiring ID Verification">
            <TextStyle variation="subdued">
              Choose which collection require ID verification. An ID
              verification link will be sent on checkout if an order contains
              one or more of the products contained in the collections below.
            </TextStyle>
            <Flex alignItems="center" my={2} mx={3}>
              <Toggle
                checked={values.requiredIdCollectionsEnabled}
                onChange={(e) => {
                  setFieldValue(
                    "requiredIdCollectionsEnabled",
                    e.target.checked
                  );
                }}
              />
              &nbsp;
              {values.requiredIdCollectionsEnabled ? (
                <>
                  Collections requiring ID verification is &nbsp;
                  <strong>active</strong>
                </>
              ) : (
                <>
                  Collections requiring ID verification is &nbsp;
                  <strong>disabled</strong>
                </>
              )}
            </Flex>
            <ResourceList
              filterControl={
                <Filters
                  filters={[]}
                  appliedFilters={[]}
                  onClearAll={() => {}}
                  queryValue={collectionQuery}
                  onQueryChange={setCollectionQuery}
                  onQueryClear={() => {
                    setCollectionQuery("");
                  }}
                  totalItemsCount={collections.length}
                />
              }
              renderItem={(collection) => {
                const imageUrl = get(collection, "image.transformedSrc");
                return (
                  <ResourceItem
                    media={
                      imageUrl ? (
                        <img src={imageUrl} />
                      ) : (
                        <Box
                          sx={{
                            ".Polaris-Icon": {
                              width: "4rem",
                              height: "4rem",
                              fill: "rgb(99, 115, 129)",
                            },
                          }}
                        >
                          <Icon source={CollectionsMajor} />
                        </Box>
                      )
                    }
                    onClick={() => {
                      // if product is already on the list, remove it
                      if (
                        values.requiredIdCollections.includes(collection.id)
                      ) {
                        const newRequiredIdCollections =
                          values.requiredIdCollections.filter(
                            (p) => p != collection.id
                          );
                        api
                          .patch(
                            "/api/shop/settings/remove-required-id-collection",
                            {
                              collectionId: collection.id,
                            }
                          )
                          .then(({ data }) => {
                            setFieldValue(
                              "requiredIdCollections",
                              data.requiredIdCollections
                            );
                            setFieldValue(
                              "requiredIdProducts",
                              data.requiredIdProducts
                            );
                            // setCollections(data.settings.requiredIdCollections);
                            // setProducts(data.settings.requiredIdProducts); // update the list and remove the
                            // setToast(
                            //   `The products in the ${collection.title} collection do not require ID verification.`
                            // );
                          })
                          .catch((err) => {
                            Sentry.captureException(err);
                            setToast(
                              "Unable to remove this collection from requiring ID verification, please contact support."
                            );
                          });
                      } else {
                        // if it is not on the list, add it
                        const newRequiredIdCollections = uniq([
                          collection.id,
                          ...values.requiredIdCollections,
                        ]);
                        // api
                        //   .patch("/api/shop/settings", {
                        //     requiredIdCollections: newRequiredIdCollections,
                        //   })
                        //   .then(({ data }) => {
                        //      setCollections(data.settings.requiredIdCollections);
                        setFieldValue(
                          "requiredIdCollections",
                          newRequiredIdCollections
                        );
                        // setToast(
                        //   `The products in the ${collection.title} collection require ID verification.`
                        // );
                        // });
                      }
                    }}
                  >
                    <Flex flexDirection="column" justifyContent="space-between">
                      {collection.title}
                      <TextStyle variation="subdued">
                        Contains {collection.productsCount} products
                      </TextStyle>
                      {collection.isIdRequired && (
                        <Box my={3}>
                          <Badge status="success">
                            ID Verification Required
                          </Badge>
                        </Box>
                      )}
                    </Flex>
                  </ResourceItem>
                );
              }}
              items={collectionsMemo}
            />
            {/* <Pagination 
                              hasPrevious
                              onPrevious={() => {

                              }}
                              hasNext
                              onNext={() => {

                              }}
                            /> */}
          </Card.Section>
        )}
      </Tabs>
    </Box>
  );
}
